import { OperationObject, ParameterLocation, ParameterObject, PathItemObject, ResponsesObject } from 'openapi3-ts';
import Ajv, { ErrorObject, Schema } from 'ajv/dist/core';
import { OperationExecutionResponse } from 'executable-openapi-types';
declare type Validate = (data: unknown) => {
    pass: boolean;
    errors?: ErrorObject[];
    data: unknown;
};
/**
 * Will return a `validate` function that'll validate some data against
 * the given schema and using the given AJV instance.
 *
 * It's a tiny wrapper around `ajv.compile` that implements a trick to
 * properly coerce and use defaults when the root data is not an
 * object.
 * https://ajv.js.org/coercion.html
 */
export declare const compileAjvBasedValidate: (ajv: Ajv, schema: Schema) => Validate;
declare type RefResolver = ($ref: string) => Promise<unknown>;
/**
 * Given `OperationObject` and `PathItemObject`, will resolve the parameters
 * specs by resolving and merging specs at both levels.
 */
export declare const resolveOperationParametersSpecs: (operationObject: OperationObject, pathItemObject: PathItemObject, resolveRef: RefResolver) => Promise<ParameterObject[]>;
export declare class InvalidParameterError extends Error {
    name: string;
    location: ParameterLocation;
    errors: ErrorObject[] | string;
    constructor(name: string, location: ParameterLocation, errors: ErrorObject[] | string);
}
export declare function formatInvalidParameterResponse(error: InvalidParameterError, _responseObject: ResponsesObject): OperationExecutionResponse;
export {};
