"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatInvalidParameterResponse = exports.InvalidParameterError = exports.resolveOperationParametersSpecs = exports.compileAjvBasedValidate = void 0;
/**
 * Will return a `validate` function that'll validate some data against
 * the given schema and using the given AJV instance.
 *
 * It's a tiny wrapper around `ajv.compile` that implements a trick to
 * properly coerce and use defaults when the root data is not an
 * object.
 * https://ajv.js.org/coercion.html
 */
const compileAjvBasedValidate = (ajv, schema) => {
    // the trick is to include the value to validate in an object (with key `ROOT`)
    // and to run the validation on this object
    const ROOT = 'ROOT';
    const validate = ajv.compile({
        type: 'object',
        additionalProperties: false,
        properties: {
            [ROOT]: schema
        }
    });
    return (data) => {
        var _a;
        const toValidate = { [ROOT]: data };
        const pass = validate(toValidate);
        // extract the new data in case it was coerced
        const newData = toValidate[ROOT];
        return {
            pass,
            errors: (_a = validate.errors) === null || _a === void 0 ? void 0 : _a.map((e) => (Object.assign(Object.assign({}, e), { 
                // we need to remove the referene to `ROOT` so that
                // it looks like we ran the validation directly on the data
                instancePath: e.instancePath.replace(`/${ROOT}/`, '/'), schemaPath: e.schemaPath.replace(`#/properties/${ROOT}/`, '#/') }))),
            data: newData
        };
    };
};
exports.compileAjvBasedValidate = compileAjvBasedValidate;
/**
 * Given `OperationObject` and `PathItemObject`, will resolve the parameters
 * specs by resolving and merging specs at both levels.
 */
const resolveOperationParametersSpecs = (operationObject, pathItemObject, resolveRef) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b;
    const pathLevelParametersSpecs = (yield Promise.all(((_a = pathItemObject.parameters) !== null && _a !== void 0 ? _a : [])
        .map((p) => __awaiter(void 0, void 0, void 0, function* () { return '$ref' in p ? yield resolveRef(p.$ref) : p; }))));
    const operationLevelParametersSpecs = (yield Promise.all(((_b = operationObject.parameters) !== null && _b !== void 0 ? _b : [])
        .map((p) => __awaiter(void 0, void 0, void 0, function* () { return '$ref' in p ? yield resolveRef(p.$ref) : p; }))));
    // operation specs override path level specs
    // https://github.com/OAI/OpenAPI-Specification/blob/main/versions/3.1.0.md#fixed-fields-7
    const parametersSpecs = pathLevelParametersSpecs;
    for (const spec of operationLevelParametersSpecs) {
        const overrideIndex = parametersSpecs.findIndex((s) => s.name === spec.name && s.in === spec.in);
        if (overrideIndex !== -1) {
            parametersSpecs[overrideIndex] = spec;
        }
        else {
            parametersSpecs.push(spec);
        }
    }
    return parametersSpecs;
});
exports.resolveOperationParametersSpecs = resolveOperationParametersSpecs;
// WIP https://git.io/Jymr8
class InvalidParameterError extends Error /* HTTError */ {
    constructor(name, location, errors) {
        super(`Parameter '${name}' in '${location}' in  is not valid`);
        this.name = name;
        this.location = location;
        this.errors = errors;
    }
}
exports.InvalidParameterError = InvalidParameterError;
function formatInvalidParameterResponse(error, _responseObject) {
    const message = typeof error.errors === 'string' ? error.errors : error.errors.map((e) => e.message).join(' and ');
    return {
        status: 400,
        content: {
            'application/json': {
                in: error.location,
                name: error.name,
                message
            }
        }
    };
}
exports.formatInvalidParameterResponse = formatInvalidParameterResponse;
//# sourceMappingURL=index.js.map